```@meta
CurrentModule = Oscar
```

```@setup oscar
using Oscar
```

```@contents
Pages = ["intro.md"]
```

# Introduction

We start our discussion of PBW-algebras by recalling their definition.
Let $K$ be a field. Given a set of indeterminates $x=\{x_1, \ldots, x_n\},$ we write
${\left\langle {x}\right\rangle}:=\langle x_{1},\ldots, x_{n} \rangle$ for the *free monoid* on $x$.
That is, the elements of $\langle x \rangle$ are the words in the finite alphabet $x$, multiplication
means concatination of words, and the identity element is the empty word.
The *free associative $K$-algebra* generated by $x_{1},\dots, x_{n}$ is the corresponding monoid algebra

$K \langle {x}\rangle:= K \langle x_{1},\dots, x_{n} \rangle.$

We consider quotients of type $A = K\langle x_1, \dots, x_n \rangle/J$, for some $n$ and some
two-sided ideal $J$ of $K\langle x_1, \dots, x_n \rangle$. In case $J$ is given by a finite set
of two-sided generators $g_1, \dots, g_r$, we say that *$A$ is generated by $x_1, \dots, x_n$
subject to the relations $g_1, \dots, g_r$*, and write

$A = K\langle x_1, \dots , x_n \mid g_k=0, \ 1\leq k \leq r \rangle.$

Working with Gröbner bases requires that we take monomial orderings into account (see the section
on Gröbner bases in the commutative algebra chapter for monomial orderings). In our context here, we use the following notation.
A *standard monomial* in $x$ is a word of type $x^\alpha=x_{1}^{\alpha_{1}}\cdots x_{n}^{\alpha_{n}},$
where $\alpha=(\alpha_1,\dots,\alpha_n)\in\mathbb N^n$. A *standard polynomial* in $K \langle x \rangle$
is a $K$-linear combination of standard monomials. Identifying the set of standard monomials in $x$ with the
usual set of monomials $\text{Mon}_n(x)$,  we may think of each global monomial ordering $>$ on the latter set
as a (total) well-ordering $>$ on the former one. Given a standard polynomial $0\neq f \in K \langle x \rangle,$
it makes, then, sense to speak of the *leading monomial* $\text{LM}_>(f)$ of $f$.

**Definition.**  ``\;`` Let $A$ be a $K$-algebra of type
```math
A = K\langle x_1, \dots , x_n \mid x_jx_i = c_{ij} \cdot x_ix_j+d_{ij},  \ 1\leq i<j \leq n \rangle,
```
where the $c_{ij}\in K$ are nonzero scalars and the $d_{ij}\in K\langle x_1, \dots , x_n\rangle$ are 
standard polynomials. Then $A$ is called a *$PBW$-algebra* if the following two conditions hold:

(1) ``\;`` There exists a global monomial ordering $>$ on $\text{Mon}_n(x)$ such that

$d_{ij}=0\ \text{ or }\ x_ix_j> \text{LM}_>(d_{ij})\ \text{ for all }\ 1\leq i<j \leq n.$

(2) ``\;`` The standard monomials represent a $K$-basis for $A$.  We then refer to this basis as a *PBW-basis* for $A$. 

Every ordering as in (1) is called *admissible* for $A$.

Given a PBW-algebra $A$ as above, we sometimes abuse our notation by denoting the class of the standard monomial $x^{\alpha}$ in $A$ also by $x^{\alpha}$,
and refer to $x^{\alpha}$ as a *standard monomial* in $A$. As these monomials form a $K$-basis for $A$, every element $0\neq f\in A$ has a unique representation
```math
f=\sum c_{\alpha}x^{\alpha}, \; \text{ with nonzero coefficients } \; c_{\alpha}\in K.
```
We refer to this representation as the *standard representation* of $f$, with *coefficients* $c_{\alpha}$, and *exponents* $\alpha$.

!!! note
    PBW-algebras are also known as G-algebras or algebras of solvable type. See Remark 1 in [LS03](@cite) for a brief historical account.
 
**Example.** ``\;`` The *$n$-th Weyl algebra over $K$* is the PBW-algebra
```math
D_n(K)=K \langle x_1,\ldots, x_n, \partial _1,\dots \partial _n \mid \partial_i x_i=x_i\partial _i +1, \partial _i x_j=x_j \partial _i \ \text { for }\ i\neq j\rangle.
```
Here,  we tacitly assume that $x_j x_i=x_i x _j$ and $\partial _j \partial_i=\partial_i \partial _j$ for all $i,j$.
Note that any  global monomial ordering on $\text{Mon}_{2n}(x, \partial)$ is admissible for $D_n(K)$.

**Proposition.**  ``\;`` Let $A$ be a PBW-algebra. Then:
- ``A`` is an integral domain,
- ``A`` is (left and right) Noetherian.

